function fit = pla_fit(data,param)

ITHDRAW = ceil(rand(1)*size(data.nelements,2));

x = param.global.samplelengthsnm;

nelements        = data.nelements;
meanthetasquared = data.meanthetasquared;
meanthetaquad    = data.meanthetaquad;
meancos          = data.meancos;
meanrsquared     = data.meanrsquared;
meanrquad        = data.meanrquad;
D4               = meanrquad - meanrsquared.^2;

%###############################################################################
% determine standard deviation: Note this the standard deviation of
% the different draws and as such not (completely) independent.
%###############################################################################
[dum , fit.meanthetasquared.std] = std_weighted(meanthetasquared,nelements);
[dum , fit.meanthetaquad   .std] = std_weighted(meanthetaquad   ,nelements);
[dum , fit.meancos         .std] = std_weighted(meancos         ,nelements);
[dum , fit.meanrsquared    .std] = std_weighted(meanrsquared    ,nelements);
[dum , fit.meanrquad       .std] = std_weighted(meanrquad 	,nelements);
fit.nelements       .std         = std(                          nelements);

%###############################################################################
% use the ITHDraw draw as supplier of the mean
%###############################################################################
fit.meanthetasquared.mean = meanthetasquared(ITHDRAW,:);
fit.meanthetaquad   .mean = meanthetaquad   (ITHDRAW,:);
fit.meancos         .mean = meancos         (ITHDRAW,:);
fit.meanrsquared    .mean = meanrsquared    (ITHDRAW,:);
fit.meanrquad       .mean = meanrquad       (ITHDRAW,:);
fit.nelements       .mean = nelements       (ITHDRAW,:);
fit.D4              .mean = D4              (ITHDRAW,:);

%###############################################################################
% measures fro which propagation of errors is neccessary to
% determine the standard deviation
%###############################################################################
%fit.rquadminmeanrsquaredsquared.mx = (fit.meanrquad.mean - fit.meanrsquared.mean.^2);

fit.cov22div4.std = cov_weighted(meanthetasquared,...
    meanthetaquad,nelements);

fit.st22div4      = sigma22div4(fit.meanthetasquared.mean,...
    fit.meanthetaquad.mean,...
    fit.meanthetasquared.std,...
    fit.meanthetaquad.std,...
    fit.cov22div4.std);

fit.st4div22=sigma4div22(fit.meanthetasquared.mean,...
    fit.meanthetaquad.mean,...
    fit.meanthetasquared.std,...
    fit.meanthetaquad.std,...
    fit.cov22div4.std);


fit.mx22div4      = fit.meanthetasquared.mean.^2./fit.meanthetaquad.mean;


fit.covR4minR2    = cov_weighted(data.meanrquad,...
    data.meanrsquared,nelements);

fit.stR4minR22    = sigmaR4minR22(fit.meanrsquared.mean,...
    fit.meanrquad.mean,...
    fit.meanrsquared.std,...
    fit.meanrquad.std,...
    fit.covR4minR2);

fit.D4.std = fit.stR4minR22;
fit.theta22divtheta4.mean = fit.mx22div4;
fit.theta22divtheta4.std  = fit.st22div4;

fit.theta4divtheta22.mean = 1./fit.mx22div4;
fit.theta4divtheta22.std  = fit.st4div22;



%###############################################################################
% actual persinstence length fitting
%###############################################################################
for cc = 1:length(param.global.samplelengthsnm)
    for bb = 1:param.global.nrandomdraws
        %    fit.perslength.r2    (bb,cc) = fminbnd(@(p) sum(((meanr2(p,x(1:cc))-data.meanrsquared(bb,1:cc))).^2)        ,0,x(end));
        fit.perslength.r2    (bb,cc) = fminbnd(@(p) meanr2f(p,x(1:cc), data.meanrsquared(bb,1:cc)),0,x(end));
        fit.perslength.cos   (bb,cc) = fminbnd(@(p) sum(((meancosf(p,x(1:cc)+1,data.meancos(bb,1:cc)))).^2)           ,0,x(end));
        fit.perslength.p2    (bb,cc) = fminbnd(@(p) sum(((meantheta2f(p,x(1:cc)+1,data.meanthetasquared(bb,1:cc)))).^2),0,x(end));
        fit.perslength.d42d  (bb,cc) = fminbnd(@(p) sum(((lsqmeanr4minmeanr22dim2df  (p,x(1:cc)+1,D4(bb,1:cc)))).^2)  ,0,x(end));
        fit.perslength.d43d  (bb,cc) = fminbnd(@(p) sum(((lsqmeanr4minmeanr22dim3df  (p,x(1:cc)+1,D4(bb,1:cc)))).^2)  ,0,x(end));
        fit.perslength.d4n2d (bb,cc) = fminbnd(@(p) sum(((lsqmeanr4minmeanr22ndim2df (p,x(1:cc)+1,D4(bb,1:cc)))).^2)  ,0,x(end));
        fit.perslength.d4n3d (bb,cc) = fminbnd(@(p) sum(((lsqmeanr4minmeanr22ndim3df (p,x(1:cc)+1,D4(bb,1:cc)))).^2)  ,0,x(end));
        fit.perslength.r42d  (bb,cc) = fminbnd(@(p) sum((((meanr42D(p,x(1:cc)+1) -data.meanrquad(bb,1:cc)))     ).^2)  ,0,x(end));
        fit.perslength.r43d  (bb,cc) = fminbnd(@(p) sum((((meanr43D(p,x(1:cc)+1) -data.meanrquad(bb,1:cc)))     ).^2)  ,0,x(end));
    end
end

%###############################################################################
% avarage persistence length over all sample windowsin respectively
% pixels and nanometer
%###############################################################################
fit.plength.nm.r2.m    = fit.perslength.r2    (ITHDRAW,:); %ITHDRAW
fit.plength.nm.cos.m   = fit.perslength.cos   (ITHDRAW,:);
fit.plength.nm.p2.m    = fit.perslength.p2    (ITHDRAW,:);
fit.plength.nm.d42d.m  = fit.perslength.d42d  (ITHDRAW,:);
fit.plength.nm.d43d.m  = fit.perslength.d43d  (ITHDRAW,:);
fit.plength.nm.d4n2d.m = fit.perslength.d4n2d (ITHDRAW,:);
fit.plength.nm.d4n3d.m = fit.perslength.d4n3d (ITHDRAW,:);
fit.plength.nm.r42d.m  = fit.perslength.r42d  (ITHDRAW,:);
fit.plength.nm.r43d.m  = fit.perslength.r43d  (ITHDRAW,:);

%###############################################################################
fit.plength.nm.r2.s    = std(fit.perslength.r2    ,[],1)/sqrt(param.global.nrandomdraws);
fit.plength.nm.cos.s   = std(fit.perslength.cos   ,[],1)/sqrt(param.global.nrandomdraws);
fit.plength.nm.p2.s    = std(fit.perslength.p2    ,[],1)/sqrt(param.global.nrandomdraws);
fit.plength.nm.d42d.s  = std(fit.perslength.d42d  ,[],1)/sqrt(param.global.nrandomdraws);
fit.plength.nm.d43d.s  = std(fit.perslength.d43d  ,[],1)/sqrt(param.global.nrandomdraws);
fit.plength.nm.d4n2d.s = std(fit.perslength.d4n2d ,[],1)/sqrt(param.global.nrandomdraws);
fit.plength.nm.d4n3d.s = std(fit.perslength.d4n3d ,[],1)/sqrt(param.global.nrandomdraws);
fit.plength.nm.r42d.s  = std(fit.perslength.r42d  ,[],1)/sqrt(param.global.nrandomdraws);
fit.plength.nm.r43d.s  = std(fit.perslength.r43d  ,[],1)/sqrt(param.global.nrandomdraws);



%###############################################################################
%
%###############################################################################

function [mx,st]=std_weighted(x,n)
%calculate weighted standar deviation: weighted as the different
%draws have different number of elements, i.e. some draws are more
%trust worthy than others
%
%
normalisedweights = (n./repmat(sum(n),[size(x,1),1]));

%calculate weighted mean over all draws for all lengths
mx = repmat(sum(x.*normalisedweights),[size(x,1),1]);

%calculate weighted std over all draws for all lengths
st = sqrt( sum( ((x-mx).^2).*normalisedweights) );

mx = mx(1,:);

%########################################################################

function st = cov_weighted(x,y,n)
normalisedweights = (n ./ repmat( sum( n ) , [ size(x,1) , 1 ] ) );
mx = repmat( sum( x .* normalisedweights ) , [ size( x , 1 ) , 1 ] );
my = repmat( sum( y .* normalisedweights ) , [ size( x , 1 ) , 1 ] );
st = sqrt( sum( (x - mx) .* (y-my) .* normalisedweights ) );

%########################################################################
% standard deviation of <theta^2>^2 / <theta^4>

% error propagation on f(a,b) = a^2 / b
% sigma_f^2 = dfda^2 *sigma_a^2 + dfdb^2 *sigma_b^2 + dfdb*dfdb *sigma_ab^2

function sig=sigma22div4(mt2,mt4,st2,st4,st2t4)
%meanthetasquared.mx,meanthetaquad.mx,meanthetasquared.st,meanthetaquad.st,cov22div4.st
sig=sqrt( ( ( 4.*(mt2.^2) ) ./ ( mt4.^2 )) .* ( st2 .^2 )...
    +( (     mt2.^4  ) ./ ( mt4.^4 )) .* ( st2 .^2 )...
    -(2.*(mt2.^3))./(mt4.^3).*(st2t4.^2));

%########################################################################
 
function sig=sigma4div22(mt2,mt4,st2,st4,st2t4)

%x,y,sx,sy,sxy)
%meanthetasquared.mx,meanthetaquad.mx,meanthetasquared.st,meanthetaquad.st,cov22div4.st
sig=sqrt(...
 (4*(mt4.^2)./(mt2.^6)).*(st2 .^2)...
+(      1./(mt2.^4)).*(st4 .^2)...
-(2*(mt4  )./(mt2.^5)).*(st2t4.^2));

%########################################################################
% standard deviation of <r^4> - <r^2>^2 , when normalising with
% contour length use L^-4 as normalisation in mean as well as sigma

% error propagation on f(a,b) = a - b^2
% sigma_f^2 = dfda^2 *sigma_a^2 + dfdb^2 *sigma_b^2 + dfdb*dfdb *sigma_ab^2

function sig=sigmaR4minR22(mr2,mr4,sr2,sr4,sr4r2)
%meanrsquared.mx,meanrquad.mx, meanrsquared.st,meanrquad.st,covR4minR2
sig=sqrt(                        (sr4 .^2)...
    +4.*(mr2.^2).*(sr2 .^2)...
    -2.*(mr2   ).*(sr4r2.^2));
%########################################################################

function f = meanr2f(q,x,y)
f =sum((meanr2(q,x)-y).^2);

%########################################################################

function f = meancosf(q,x,y)
f =(avgcos(q,x)-y);

%########################################################################

function f = avgcos(q,x)
f =exp(-x./(2*q));

%########################################################################

function f = meantheta2f(q,x,y)
f = ((x./q)-y);

%########################################################################

function f = meanr2(P,L)
f =(4.*P.*L-8.*P.^2.*(1-exp(-L./(2.*P))));

%########################################################################

function f = meanr2_3D(P,L)
f =(2*P.*L-2.*P.^2.*(1-exp(-L./P)));

%########################################################################


function f = meanr42D(P,L)
P2D=P*2;
f=(    8   .*L.^2.*P2D.^2 ...
    - 30   .*L   .*P2D.^3 ...
    + 87/2 .*      P2D.^4 ...
    - 40/3 .*L   .*P2D.^3.*exp(-   L./P2D) ...
    -392/9 .*      P2D.^4.*exp(-   L./P2D) ...
    +  1/18.*      P2D.^4.*exp(-4.*L./P2D));

%########################################################################

function f = meanr43D(P,L)
f=(20/3 .*L.^2.*P.^2 ...
    -208/9 .*L   .*P.^3 ...
    +856/27.*      P.^4 ...
    -8     .*L   .*P.^3.*exp(-   L./P) ...
    -32    .*      P.^4.*exp(-   L./P) ...
    +8/27 .*      P.^4.*exp(-3.*L./P));

%########################################################################
%########################################################################


%########################################################################
% 2D function for D^4, NOT normalised for L
% D^4 = <R^4> -<r^2>^2

function f = lsqmeanr4minmeanr22dim2df(P,L,y)
f = (meanr4minmeanr22dim2d(P,L)-mean(y,1));

function f= meanr4minmeanr22dim2d(P,L)
f = (meanr42D(P,L)-meanr2(P,L).^2);

%########################################################################

%########################################################################
% 3D function for D^4, NOT normalised for L
% D^4 = <R^4> -<r^2>^2

function f = lsqmeanr4minmeanr22dim3df(P,L,y)
f = (meanr4minmeanr22dim3d(P,L)-mean(y,1));

function f= meanr4minmeanr22dim3d(P,L)
f = (meanr43D(P,L)-meanr2_3D(P,L).^2);

%########################################################################

%########################################################################
% 2D function for D^4, normalised for L
% D^4 = <R^4> -<r^2>^2

function f = lsqmeanr4minmeanr22ndim2df(P,L,y)
f = sqrt(((meanr4minmeanr22dim2d(P,L)-mean(y,1)).^2))./L.^4;

function f= meanr4minmeanr22ndim2d(P,L)
f = (meanr42D(P,L)-meanr2(P,L).^2);

%########################################################################

%########################################################################
% 3D function for D^4, normalised for L
% D^4 = <R^4> -<r^2>^2

function f = lsqmeanr4minmeanr22ndim3df(P,L,y)
f = sqrt(((meanr4minmeanr22dim3d(P,L)-mean(y,1)).^2))./L.^4;

function f= meanr4minmeanr22ndim3d(P,L)
f = (meanr43D(P,L)-meanr2_3D(P,L).^2);

%########################################################################


