function param = pla_settings
% filename convention:
% image filenames::  condition_num.extension
% e.g. ["DNA_image_L474_P55_segL1" "_" "11" "." "ics"]
% as such a single number, e.g. 10, between the last underscore and the
% extension period is seen as 10th image of the condition. Gaps in
% the indexing are allowed, e.g. conditionA_1.ics,conditionA_2.ics,conditionA_11.ics


demo=true;

if ~demo
    %output directory
    param.dir.output = 'somedirectory of your choice';
    
    %image file directory
    param.dir.images = 'somedirectory of your choice';
else
    basedir = fileparts([fileparts(mfilename('fullpath')) '.dum']);

    %output directory
    param.dir.output = [basedir filesep 'results'];
    
    %image file directory
    param.dir.images = [basedir filesep 'data'];
end

%info file directory
param.dir.info   = param.dir.images;

%extension of images
param.extension.images ='tif';

%extension of infofile
param.extension.info = 'mat';

%extension of label images
param.extension.label ='ics';

%cutoff (px) of strands to correct for troublesome track ends
param.global.cutoff = 3;

%at which lengths should the distribution be sampled
param.global.samplelengthsnm = [10:10:100];

%how many random draws are required to get a estimate of the
%standard deviation of the fits and data points themselves
param.global.nrandomdraws    = 21;

%
param.global.cutoff = 3;
param.global.samplelengthsnm = [10:10:100];
param.global.nrandomdraws    = 21;

param.global.dekker.samplelengthsnm = [2:2:8];
param.global.dekker.bins = linspace(0,pi/2,15);
param.global.maxphi = pi/2;
param.experiment = 'experiment';

%get settings either based on previous declared param or older
%settings file
param = getsettings(param);

%%

function param = group_files(param)
%
% group files with same condition
%

filenamelist = dir([param.dir.images filesep '*.' param.extension.info]);

if(size(filenamelist,1)==0)
    fprintf('no files found\n');
else
    %get conditions for each file
    fprintf('\nget setting for each file\n');
    for ii=1:size(filenamelist,1)
        fprintf('   %3d / %3d\n',ii,size(filenamelist,1));
        condtmp{ii} = getcondition(filenamelist(ii).name);
    end
end

%group conditions
disp('group conditions')
lab = zeros(size(filenamelist));
cnt=1;
while any(lab==0)
    ind = find(lab==0);
    unq = zeros(size(ind));
    
    for ii = 1:length(ind)
        %  condtmp{ind(1)}
        unq(ii) = strcmp(condtmp{ind(1)},condtmp{ind(ii)});
    end
    %  sum(unq)
    lab(ind(find(unq))) = cnt;
    cnt=cnt+1;
end

labs = unique(lab);

for ii = 1:length(labs)
    fprintf('   %3d / %3d\n',ii,length(labs));
    ind = find(lab == labs(ii));
    condition(ii).name  = condtmp(ind(1));
    
    condition(ii).results = [param.dir.output filesep condition(ii).name{1} ,'/'];
    if(~(exist(condition(ii).results,'dir')==7))
        mkdir(condition(ii).results)
    end
    
    for jj = 1 : length(ind)
        condition(ii).files{jj} = filenamelist(ind(jj)).name;
        [dummy,fn]=fileparts(condition(ii).files{jj});
        load([param.dir.info filesep  fn '.' param.extension.info])
        condition(ii).param{jj}.NM_per_PX = infoimage.NM_per_PX;
        condition(ii).param{jj}.BP_per_PX = infoimage.BP_per_PX;
        condition(ii).param{jj}.NM_per_BP = infoimage.NM_per_BP;
        condition(ii).param{jj}.backgroundisblack = infoimage.backgroundisblack;
        %    condition(ii).param{jj}.crossings = infoimage.crossings;
        if (isfield(infoimage,'posA') && ...
                isfield(infoimage,'posB') && ...
                length(size(infoimage.posA))==2 &&...
                length(size(infoimage.posB))==2 &&...
                ~any(~(size(infoimage.posA)==size(infoimage.posB))))
            condition(ii).param{jj}.posA     = infoimage.posA;
            condition(ii).param{jj}.posB     = infoimage.posB;
        end
    end
end

param.condition = condition;

%%
function condition=getcondition(str)


%find num
tmp = regexp(str,'_([0-9]*)?\.\w+','tokens');
if ~isempty(tmp) && iscell(tmp) && length(size(tmp))==2 && sum(size(tmp)==[1,1])==2
    num = ['_' tmp{1}{1}];
else
    num = [];
end

%find extension
tmp = regexp(str,'\w+\.(\w+)','tokens');
if ~isempty(tmp) && iscell(tmp) && length(size(tmp))==2 && sum(size(tmp)==[1,1])==2
    ext = tmp{1}{1};
else
    error(['no extension detected in ' , mfilename]);
end

%find condition
tmp = regexp(str,['(\w+)' [num '.' ext]],'tokens');
if ~isempty(tmp) && iscell(tmp) && length(size(tmp))==2 && sum(size(tmp)==[1,1])==2
    condition = tmp{1}{1};
else
    error(['no condition detected in ' , mfilename]);
end

%%

function param = getsettings(param)

%check if directory exist
if(~(exist(param.dir.output,'dir')==7))
    disp('create output directry');
    mkdir(param.dir.output)
else
    fprintf('output directry already exists\n');
end

%name output setting file
outfile = [param.dir.output filesep 'settings.mat' ];
if(exist(outfile,'file'))
    clear('param');
    load(outfile,'param');
    fprintf('older settings file found\n');
    fprintf('   using:: %s\n', outfile);
    fprintf('   all previous set parameters are replaced by parameters in settings file\n');
else
    fprintf('create new settings file\n');
    param = group_files(param);
    save(outfile,'param');
end
%%

